@@ -1,5 +1,4 @@
 #![feature(fundamental)]
-#![feature(re_rebalance_coherence)]
 
 // compile-flags:--crate-name=test
 // aux-build:coherence_lib.rs
@@ -9,8 +8,8 @@ extern crate coherence_lib as lib;
 use lib::*;
 
 #[fundamental]
-struct Local;
+struct Local<T>(T);
 
-impl Remote for Local {}
+impl Remote for Local<()> {}
 
 fn main() {}
